(**
   This is a module on ABH transformation.
@author Naoki Nishida
@since 2017/01/31
*)

open Ctrs;;


exception Condition1;;
exception Condition2;;
   
(**
   This works only for normal CTRSs where each rule has at most one condition.
*)
let not_preserve_SN rs0 =
  try
    let rec decompose (urs0,cs0) =
      function
	  [] -> (urs0,cs0)
	| rl0::rs0 ->
	    begin
	      match rl0 with
		Rew(l,r,[]) ->
		  if is_var r
		  then		      
		    decompose ((l,r)::urs0,cs0) rs0
		  else
		    decompose (urs0,cs0) rs0
	      | Rew(l,r,[Arrow(s,t)]) ->
		  if is_var s
		  then
		    raise Condition2
		  else
		    decompose (urs0,(s,t)::cs0) rs0
	      | _ -> raise (Failure "unexpected")
	    end
    in
    let rec satisfy_condition1 (l,r) (s,_) =
      try
	let delta = Rename.gen_renaming (vars_in_term l) in
	let sigma = unify l (apply_subst_to_term delta s) in
	is_var (apply_subst_to_term sigma r)
      with
	Not_found -> false
    in
    let verify_condition1 cs0 (l,r) =
      List.exists (satisfy_condition1 (l,r)) cs0
    in
    let (urs,cs) = decompose ([],[]) rs0 in
    if List.exists (verify_condition1 cs) urs
    then
      raise Condition1
    else
      false
  with
    Condition1 -> true
  | Condition2 -> true;;

(**
   This works only for normal CTRSs where each rule has at most one condition.
*)
(*
 let test spec =
  let cond2 = ref false in
  let rs0 = List.map Convert_NormalOneCTRS.mk_singly_conditional spec.rules
  in
  try
    let rec decompose (urs0,cs0) =
      function
	  [] -> (urs0,cs0)
	| rl0::rs0 ->
	    begin
	      match rl0 with
		Rew(l,r,[]) ->
		  if is_var r
		  then		      
		    decompose ((l,r)::urs0,cs0) rs0
		  else
		    decompose (urs0,cs0) rs0
	      | Rew(l,r,[Arrow(s,t)]) ->
		  if is_var s
                  then
                    begin
                      cond2 := true;
                      decompose (urs0,(s,t)::cs0) rs0
                    end
		  else
		    decompose (urs0,(s,t)::cs0) rs0
	      | _ -> raise (Failure "unexpected")
	    end
    in
    let rec satisfy_condition1 (l,r) (s,_) =
      try
	let delta = Rename.gen_renaming (vars_in_term l) in
	let sigma = unify l (apply_subst_to_term delta s) in
	is_var (apply_subst_to_term sigma r)
      with
	Not_found -> false
    in
    let verify_condition1 cs0 (l,r) =
      List.exists (satisfy_condition1 (l,r)) cs0
    in
    let (urs,cs) = decompose ([],[]) rs0 in
    if List.exists (verify_condition1 cs) urs
    then
      raise Condition1
    else
      begin
        if !cond2
        then
          print_string "NO (by Condition 2)\n"
        else
          print_string "MAYBE\n"
      end
  with
    Condition1 ->
      begin
        if !cond2
        then
          print_string "NO (by Conditions 1 & 2)\n"
        else
          print_string "NO (by Condition 1)\n"
      end
  | Condition2 ->
      print_string "NO (by Condition 2)\n";;
*)
