type 'a graph = 
    { nodes : (int * 'a) list;
      edges : (int * int) list;
    } ;;

let mk_graph ns arc =
  let rec indexing n =
    function
	[] -> []
      |	(x::xs) -> (n,x)::(indexing (n+1) xs)
  in
  let rec mk_edge0 (n,v) =
    function
	[] -> []
      |	((n1,v1)::ns1) ->
	  let es1 = mk_edge0 (n,v) ns1 in
	  begin
	    match (arc (v,v1), arc (v1,v)) with
	      (true,true) -> (n,n1)::(n1,n)::es1
	    | (false,true) -> (n1,n)::es1
	    | (true,false) -> (n,n1)::es1
	    | (false,false) -> es1
	  end
  and mk_edge =
    function
	[] -> []
      |	((n,v)::ns) ->
	  if arc (v,v)
	  then (n,n)::((mk_edge0 (n,v) ns)@(mk_edge ns))
	  else (mk_edge0 (n,v) ns) @ (mk_edge ns)
  in
  let ns1 = indexing 1 ns in
  let es1 = mk_edge ns1 in
  let ns2 = List.filter (fun (x,_) -> List.mem_assoc x es1) ns1 in
  let es2 = mk_edge ns2 in
  { nodes = ns2;
    edges = es2;
  };;

(*
let all_next g n =
  let rec all_next0 =
    function
	[] -> []
      |	((n1,n2)::es) ->
	  if n = n1 then n2::(all_next0 es) else all_next0 es
  in
  all_next0 g.edges;;
*)

let union_SCC g =
  let self_reachable n =
    let finished = ref [] in
    let rec next n1 =
      function
	  [] -> []
	| (n2,n3)::es ->
	    if n1 = n2
	    then
	      n3::(next n1 es)
	    else
	      next n1 es
    and check ns =
      begin
	match ns with
	  [] -> false
	| (n1::ns1) ->
	    begin
	      (List.mem n ns)
	      (*
	      if List.mem n ns
	      then
		begin
		  print_string "(COMMENT ";
		  print_string "CYCLE: ";
		  print_string ("("^(string_of_int n)^") -*-> ("^(string_of_int n)^") )\n");
		  true
		end
	      else
		false
		*)
	    ||
	      begin
		if List.mem n1 (!finished)
		then
		  check ns1
		else
		  let ns2 = next n1 g.edges in
		  begin
		    finished := n1::(!finished);
		    check ns2
		end
	      end
	    end
      end
    in
    check (next n g.edges)
  in
  begin
    try
      List.map
	(fun n -> List.assoc n g.nodes)
	(List.filter self_reachable (List.map (fun (n,_) -> n) g.nodes))
    with 
      Not_found -> raise (Failure "Error in graph.union_scc: unexpected\n")
  end;;
